@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2010 by Digital Mars
+// Copyright (c) 1999-2011 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -43,13 +43,13 @@ void obj_startaddress(Symbol *s);
 
 /********************************* AttribDeclaration ****************************/
 
-AttribDeclaration::AttribDeclaration(Array *decl)
+AttribDeclaration::AttribDeclaration(Dsymbols *decl)
         : Dsymbol()
 {
     this->decl = decl;
 }
 
-Array *AttribDeclaration::include(Scope *sc, ScopeDsymbol *sd)
+Dsymbols *AttribDeclaration::include(Scope *sc, ScopeDsymbol *sd)
 {
     return decl;
 }
@@ -57,12 +57,13 @@ Array *AttribDeclaration::include(Scope *sc, ScopeDsymbol *sd)
 int AttribDeclaration::addMember(Scope *sc, ScopeDsymbol *sd, int memnum)
 {
     int m = 0;
-    Array *d = include(sc, sd);
+    Dsymbols *d = include(sc, sd);
 
     if (d)
     {
         for (unsigned i = 0; i < d->dim; i++)
-        {   Dsymbol *s = (Dsymbol *)d->data[i];
+        {   Dsymbol *s = d->tdata()[i];
+            //printf("\taddMember %s to %s\n", s->toChars(), sd->toChars());
             m |= s->addMember(sc, sd, m | memnum);
         }
     }
@@ -92,7 +93,7 @@ void AttribDeclaration::setScopeNewSc(Scope *sc,
             newsc->structalign = structalign;
         }
         for (unsigned i = 0; i < decl->dim; i++)
-        {   Dsymbol *s = (Dsymbol *)decl->data[i];
+        {   Dsymbol *s = decl->tdata()[i];
 
             s->setScope(newsc); // yes, the only difference from semanticNewSc()
         }
@@ -127,7 +128,7 @@ void AttribDeclaration::semanticNewSc(Scope *sc,
             newsc->structalign = structalign;
         }
         for (unsigned i = 0; i < decl->dim; i++)
-        {   Dsymbol *s = (Dsymbol *)decl->data[i];
+        {   Dsymbol *s = decl->tdata()[i];
 
             s->semantic(newsc);
         }
@@ -141,14 +142,14 @@ void AttribDeclaration::semanticNewSc(Scope *sc,
 
 void AttribDeclaration::semantic(Scope *sc)
 {
-    Array *d = include(sc, NULL);
+    Dsymbols *d = include(sc, NULL);
 
     //printf("\tAttribDeclaration::semantic '%s', d = %p\n",toChars(), d);
     if (d)
     {
-        for (unsigned i = 0; i < d->dim; i++)
+        for (size_t i = 0; i < d->dim; i++)
         {
-            Dsymbol *s = (Dsymbol *)d->data[i];
+            Dsymbol *s = d->tdata()[i];
 
             s->semantic(sc);
         }
@@ -157,12 +158,12 @@ void AttribDeclaration::semantic(Scope *sc)
 
 void AttribDeclaration::semantic2(Scope *sc)
 {
-    Array *d = include(sc, NULL);
+    Dsymbols *d = include(sc, NULL);
 
     if (d)
     {
-        for (unsigned i = 0; i < d->dim; i++)
-        {   Dsymbol *s = (Dsymbol *)d->data[i];
+        for (size_t i = 0; i < d->dim; i++)
+        {   Dsymbol *s = d->tdata()[i];
             s->semantic2(sc);
         }
     }
@@ -170,12 +171,12 @@ void AttribDeclaration::semantic2(Scope *sc)
 
 void AttribDeclaration::semantic3(Scope *sc)
 {
-    Array *d = include(sc, NULL);
+    Dsymbols *d = include(sc, NULL);
 
     if (d)
     {
-        for (unsigned i = 0; i < d->dim; i++)
-        {   Dsymbol *s = (Dsymbol *)d->data[i];
+        for (size_t i = 0; i < d->dim; i++)
+        {   Dsymbol *s = d->tdata()[i];
             s->semantic3(sc);
         }
     }
@@ -183,12 +184,12 @@ void AttribDeclaration::semantic3(Scope *sc)
 
 void AttribDeclaration::inlineScan()
 {
-    Array *d = include(NULL, NULL);
+    Dsymbols *d = include(NULL, NULL);
 
     if (d)
     {
         for (unsigned i = 0; i < d->dim; i++)
-        {   Dsymbol *s = (Dsymbol *)d->data[i];
+        {   Dsymbol *s = d->tdata()[i];
             //printf("AttribDeclaration::inlineScan %s\n", s->toChars());
             s->inlineScan();
         }
@@ -197,14 +198,15 @@ void AttribDeclaration::inlineScan()
 
 void AttribDeclaration::addComment(unsigned char *comment)
 {
+    //printf("AttribDeclaration::addComment %s\n", comment);
     if (comment)
     {
-        Array *d = include(NULL, NULL);
+        Dsymbols *d = include(NULL, NULL);
 
         if (d)
         {
             for (unsigned i = 0; i < d->dim; i++)
-            {   Dsymbol *s = (Dsymbol *)d->data[i];
+            {   Dsymbol *s = d->tdata()[i];
                 //printf("AttribDeclaration::addComment %s\n", s->toChars());
                 s->addComment(comment);
             }
@@ -224,12 +226,12 @@ void AttribDeclaration::emitComment(Scope *sc)
      * Hence, Ddoc omits attributes from template members.
      */
 
-    Array *d = include(NULL, NULL);
+    Dsymbols *d = include(NULL, NULL);
 
     if (d)
     {
         for (unsigned i = 0; i < d->dim; i++)
-        {   Dsymbol *s = (Dsymbol *)d->data[i];
+        {   Dsymbol *s = d->tdata()[i];
             //printf("AttribDeclaration::emitComment %s\n", s->toChars());
             s->emitComment(sc);
         }
@@ -238,12 +240,12 @@ void AttribDeclaration::emitComment(Scope *sc)
 
 void AttribDeclaration::toObjFile(int multiobj)
 {
-    Array *d = include(NULL, NULL);
+    Dsymbols *d = include(NULL, NULL);
 
     if (d)
     {
         for (unsigned i = 0; i < d->dim; i++)
-        {   Dsymbol *s = (Dsymbol *)d->data[i];
+        {   Dsymbol *s = d->tdata()[i];
             s->toObjFile(multiobj);
         }
     }
@@ -253,12 +255,12 @@ int AttribDeclaration::cvMember(unsigned char *p)
 {
     int nwritten = 0;
     int n;
-    Array *d = include(NULL, NULL);
+    Dsymbols *d = include(NULL, NULL);
 
     if (d)
     {
         for (unsigned i = 0; i < d->dim; i++)
-        {   Dsymbol *s = (Dsymbol *)d->data[i];
+        {   Dsymbol *s = d->tdata()[i];
             n = s->cvMember(p);
             if (p)
                 p += n;
@@ -270,13 +272,13 @@ int AttribDeclaration::cvMember(unsigned char *p)
 
 int AttribDeclaration::hasPointers()
 {
-    Array *d = include(NULL, NULL);
+    Dsymbols *d = include(NULL, NULL);
 
     if (d)
     {
         for (size_t i = 0; i < d->dim; i++)
         {
-            Dsymbol *s = (Dsymbol *)d->data[i];
+            Dsymbol *s = d->tdata()[i];
             if (s->hasPointers())
                 return 1;
         }
@@ -291,19 +293,19 @@ const char *AttribDeclaration::kind()
 
 int AttribDeclaration::oneMember(Dsymbol **ps)
 {
-    Array *d = include(NULL, NULL);
+    Dsymbols *d = include(NULL, NULL);
 
     return Dsymbol::oneMembers(d, ps);
 }
 
 void AttribDeclaration::checkCtorConstInit()
 {
-    Array *d = include(NULL, NULL);
+    Dsymbols *d = include(NULL, NULL);
 
     if (d)
     {
         for (unsigned i = 0; i < d->dim; i++)
-        {   Dsymbol *s = (Dsymbol *)d->data[i];
+        {   Dsymbol *s = d->tdata()[i];
             s->checkCtorConstInit();
         }
     }
@@ -314,12 +316,12 @@ void AttribDeclaration::checkCtorConstInit()
 
 void AttribDeclaration::addLocalClass(ClassDeclarations *aclasses)
 {
-    Array *d = include(NULL, NULL);
+    Dsymbols *d = include(NULL, NULL);
 
     if (d)
     {
         for (unsigned i = 0; i < d->dim; i++)
-        {   Dsymbol *s = (Dsymbol *)d->data[i];
+        {   Dsymbol *s = d->tdata()[i];
             s->addLocalClass(aclasses);
         }
     }
@@ -330,17 +332,24 @@ void AttribDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
 {
     if (decl)
     {
-        buf->writenl();
-        buf->writeByte('{');
-        buf->writenl();
-        for (unsigned i = 0; i < decl->dim; i++)
+        if (decl->dim == 0)
+            buf->writestring("{}");
+        else if (decl->dim == 1)
+            (decl->tdata()[0])->toCBuffer(buf, hgs);
+        else
         {
-            Dsymbol *s = (Dsymbol *)decl->data[i];
+            buf->writenl();
+            buf->writeByte('{');
+            buf->writenl();
+            for (unsigned i = 0; i < decl->dim; i++)
+            {
+                Dsymbol *s = decl->tdata()[i];
 
-            buf->writestring("    ");
-            s->toCBuffer(buf, hgs);
+                buf->writestring("    ");
+                s->toCBuffer(buf, hgs);
+            }
+            buf->writeByte('}');
         }
-        buf->writeByte('}');
     }
     else
         buf->writeByte(';');
@@ -349,7 +358,7 @@ void AttribDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
 
 /************************* StorageClassDeclaration ****************************/
 
-StorageClassDeclaration::StorageClassDeclaration(StorageClass stc, Array *decl)
+StorageClassDeclaration::StorageClassDeclaration(StorageClass stc, Dsymbols *decl)
         : AttribDeclaration(decl)
 {
     this->stc = stc;
@@ -486,13 +495,12 @@ void StorageClassDeclaration::stcToCBuffer(OutBuffer *buf, StorageClass stc)
 void StorageClassDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
 {
     stcToCBuffer(buf, stc);
-
     AttribDeclaration::toCBuffer(buf, hgs);
 }
 
 /********************************* LinkDeclaration ****************************/
 
-LinkDeclaration::LinkDeclaration(enum LINK p, Array *decl)
+LinkDeclaration::LinkDeclaration(enum LINK p, Dsymbols *decl)
         : AttribDeclaration(decl)
 {
     //printf("LinkDeclaration(linkage = %d, decl = %p)\n", p, decl);
@@ -535,7 +543,7 @@ void LinkDeclaration::semantic3(Scope *sc)
         sc->linkage = linkage;
         for (unsigned i = 0; i < decl->dim; i++)
         {
-            Dsymbol *s = (Dsymbol *)decl->data[i];
+            Dsymbol *s = decl->tdata()[i];
 
             s->semantic3(sc);
         }
@@ -574,7 +582,7 @@ char *LinkDeclaration::toChars()
 
 /********************************* ProtDeclaration ****************************/
 
-ProtDeclaration::ProtDeclaration(enum PROT p, Array *decl)
+ProtDeclaration::ProtDeclaration(enum PROT p, Dsymbols *decl)
         : AttribDeclaration(decl)
 {
     protection = p;
@@ -611,9 +619,9 @@ void ProtDeclaration::importAll(Scope *sc)
        newsc->explicitProtection = 1;
     }
 
-    for (int i = 0; i < decl->dim; i++)
+    for (size_t i = 0; i < decl->dim; i++)
     {
-       Dsymbol *s = (Dsymbol *)decl->data[i];
+       Dsymbol *s = (*decl)[i];
        s->importAll(newsc);
     }
 
@@ -656,7 +664,7 @@ void ProtDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
 
 /********************************* AlignDeclaration ****************************/
 
-AlignDeclaration::AlignDeclaration(unsigned sa, Array *decl)
+AlignDeclaration::AlignDeclaration(unsigned sa, Dsymbols *decl)
         : AttribDeclaration(decl)
 {
     salign = sa;
@@ -698,7 +706,7 @@ void AlignDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
 
 /********************************* AnonDeclaration ****************************/
 
-AnonDeclaration::AnonDeclaration(Loc loc, int isunion, Array *decl)
+AnonDeclaration::AnonDeclaration(Loc loc, int isunion, Dsymbols *decl)
         : AttribDeclaration(decl)
 {
     this->loc = loc;
@@ -719,6 +727,12 @@ void AnonDeclaration::semantic(Scope *sc)
 {
     //printf("\tAnonDeclaration::semantic %s %p\n", isunion ? "union" : "struct", this);
 
+    if (sem == 1)
+    {   //printf("already completed\n");
+        scope = NULL;
+        return;             // semantic() already completed
+    }
+
     Scope *scx = NULL;
     if (scope)
     {   sc = scope;
@@ -766,7 +780,7 @@ void AnonDeclaration::semantic(Scope *sc)
 
         for (unsigned i = 0; i < decl->dim; i++)
         {
-            Dsymbol *s = (Dsymbol *)decl->data[i];
+            Dsymbol *s = decl->tdata()[i];
 
             s->semantic(sc);
             if (isunion)
@@ -851,7 +865,7 @@ void AnonDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
     {
         for (unsigned i = 0; i < decl->dim; i++)
         {
-            Dsymbol *s = (Dsymbol *)decl->data[i];
+            Dsymbol *s = decl->tdata()[i];
 
             //buf->writestring("    ");
             s->toCBuffer(buf, hgs);
@@ -867,7 +881,7 @@ const char *AnonDeclaration::kind()
 
 /********************************* PragmaDeclaration ****************************/
 
-PragmaDeclaration::PragmaDeclaration(Loc loc, Identifier *ident, Expressions *args, Array *decl)
+PragmaDeclaration::PragmaDeclaration(Loc loc, Identifier *ident, Expressions *args, Dsymbols *decl)
         : AttribDeclaration(decl)
 {
     this->loc = loc;
@@ -877,7 +891,7 @@ PragmaDeclaration::PragmaDeclaration(Loc loc, Identifier *ident, Expressions *ar
 
 Dsymbol *PragmaDeclaration::syntaxCopy(Dsymbol *s)
 {
-        //printf("PragmaDeclaration::syntaxCopy(%s)\n", toChars());
+    //printf("PragmaDeclaration::syntaxCopy(%s)\n", toChars());
     PragmaDeclaration *pd;
 
     assert(!s);
@@ -897,15 +911,16 @@ void PragmaDeclaration::setScope(Scope *sc)
         }
         else
         {
-            Expression *e = (Expression *)args->data[0];
+            Expression *e = args->tdata()[0];
             e = e->semantic(sc);
             e = e->optimize(WANTvalue | WANTinterpret);
-            args->data[0] = (void *)e;
-            if (e->op != TOKstring)
+            args->tdata()[0] = e;
+            StringExp* se = e->toString();
+            if (!se)
             {
                 error("string expected, not '%s'", e->toChars());
             }
-            PragmaScope* pragma = new PragmaScope(this, sc->parent, static_cast<StringExp*>(e));
+            PragmaScope* pragma = new PragmaScope(this, sc->parent, se);
 
             assert(sc);
             pragma->setScope(sc);
@@ -930,13 +945,13 @@ void PragmaDeclaration::semantic(Scope *sc)
         {
             for (size_t i = 0; i < args->dim; i++)
             {
-                Expression *e = (Expression *)args->data[i];
+                Expression *e = args->tdata()[i];
 
                 e = e->semantic(sc);
                 e = e->optimize(WANTvalue | WANTinterpret);
-                if (e->op == TOKstring)
+                StringExp *se = e->toString();
+                if (se)
                 {
-                    StringExp *se = (StringExp *)e;
                     fprintf(stdmsg, "%.*s", (int)se->len, (char *)se->string);
                 }
                 else
@@ -952,16 +967,18 @@ void PragmaDeclaration::semantic(Scope *sc)
             error("string expected for library name");
         else
         {
-            Expression *e = (Expression *)args->data[0];
+            Expression *e = args->tdata()[0];
 
             e = e->semantic(sc);
             e = e->optimize(WANTvalue | WANTinterpret);
-            args->data[0] = (void *)e;
-            if (e->op != TOKstring)
+            args->tdata()[0] = e;
+            if (e->op == TOKerror)
+                goto Lnodecl;
+            StringExp *se = e->toString();
+            if (!se)
                 error("string expected for library name, not '%s'", e->toChars());
             else if (global.params.verbose)
             {
-                StringExp *se = (StringExp *)e;
                 char *name = (char *)mem.malloc(se->len + 1);
                 memcpy(name, se->string, se->len);
                 name[se->len] = 0;
@@ -993,10 +1010,11 @@ void PragmaDeclaration::semantic(Scope *sc)
             if (!d)
                 error("first argument of GNU_asm must be a function or variable declaration");
 
-            e = (Expression *)args->data[1];
+            e = args->tdata()[1];
             e = e->semantic(sc);
-            e = e->optimize(WANTvalue | WANTinterpret);
-            if (e->op == TOKstring && ((StringExp *)e)->sz == 1)
+            e = e->optimize(WANTvalue);
+            e = e->toString();
+            if (e && ((StringExp *)e)->sz == 1)
                 s = ((StringExp *)e);
             else
                 error("second argument of GNU_asm must be a character string");
@@ -1006,8 +1024,11 @@ void PragmaDeclaration::semantic(Scope *sc)
         }
         goto Lnodecl;
     }
-    else if (ident == Id::GNU_attribute)
+    else if (ident == Id::GNU_attribute || ident == Id::_GNU_attribute)
     {
+        if (!global.params.useDeprecated && ident == Id::_GNU_attribute)
+            error("pragma(GNU_attribute) is deprecated, use pragma(attribute) instead");
+
         sc_save = *sc;
 
         // An empty list is allowed.
@@ -1023,20 +1044,23 @@ void PragmaDeclaration::semantic(Scope *sc)
 
             for (unsigned i = 0; i < args->dim; i++)
             {
-                Expression * e = (Expression *) args->data[i];
+                Expression * e = args->tdata()[i];
                 //e = e->semantic(sc);
-
-                if (e->op == TOKidentifier) {
-                    /* ok */
-                } else if (e->op == TOKcall) {
+                
+                if (e->op == TOKidentifier)
+                    ; // ok
+                else if (e->op == TOKcall)
+                {
                     CallExp * c = (CallExp *) e;
                     if (c->e1->op != TOKidentifier)
                         error("identifier or call expression expected for attribute");
                     if (c->arguments)
                         for (int unsigned ai = 0; ai < c->arguments->dim; ai++)
                         {
-                            c->arguments->data[ai] =
-                                ((Expression *) c->arguments->data[ai])->semantic(sc);
+                            Expression * ea = c->arguments->tdata()[ai];
+                            ea = ea->semantic(sc);
+                            ea = ea->optimize(WANTvalue | WANTinterpret);
+                            c->arguments->tdata()[ai] = ea;
                         }
                 }
                 else
@@ -1048,60 +1072,63 @@ void PragmaDeclaration::semantic(Scope *sc)
             }
         }
     }
-    else if (ident == Id::GNU_set_attribute)
+    else if (ident == Id::GNU_set_attribute || ident == Id::_GNU_set_attribute)
     {
+        if (!global.params.useDeprecated && ident == Id::_GNU_set_attribute)
+            error("pragma(GNU_set_attribute) is deprecated, use pragma(set_attribute) instead");
+
         if (!args || args->dim < 1)
             error("declaration expected for setting attributes");
         else
         {
-            Array p_attributes_list; // of Expressions**
+            Expressions ** p_attributes = NULL;     // list of existing attributes
             {
-                Expression * e = (Expression *) args->data[0];
-                Expressions ** pa = NULL;
-
+                Expression * e = args->tdata()[0];
+                
                 e = e->semantic(sc);
                 if (e->op == TOKvar)
                 {
                     Declaration * d = ((VarExp *)e)->var;
                     if (d->isFuncDeclaration() || d->isVarDeclaration())
-                        pa = & d->attributes;
+                        p_attributes = & d->attributes;
                 }
                 else if (e->op == TOKtype)
                 {
                     Type * t = ((TypeExp *)e)->type;
                     if (t->ty == Ttypedef)
-                        pa = & ((TypeTypedef *) t)->sym->attributes;
+                        p_attributes = & ((TypeTypedef *) t)->sym->attributes;
                     else if (t->ty == Tenum)
-                        pa = & ((TypeEnum *) t)->sym->attributes;
+                        p_attributes = & ((TypeEnum *) t)->sym->attributes;
                     else if (t->ty == Tstruct)
-                        pa = & ((TypeStruct *) t)->sym->attributes;
+                        p_attributes = & ((TypeStruct *) t)->sym->attributes;
                     else if (t->ty == Tclass)
-                        pa = & ((TypeClass *) t)->sym->attributes;
+                        p_attributes = & ((TypeClass *) t)->sym->attributes;
                 }
 
-                if (pa)
-                    p_attributes_list.push(pa);
-                else
+                if (p_attributes == NULL)
                     error("first argument must be a function, variable, or type declaration");
             }
 
             Expressions * new_attrs = new Expressions;
             for (unsigned i = 1; i < args->dim; i++)
             {
-                Expression * e = (Expression *) args->data[i];
+                Expression * e = args->tdata()[i];
                 //e = e->semantic(sc);
-
-                if (e->op == TOKidentifier) {
-                    /* ok */
-                } else if (e->op == TOKcall) {
+                
+                if (e->op == TOKidentifier)
+                    ; // ok
+                else if (e->op == TOKcall)
+                {
                     CallExp * c = (CallExp *) e;
                     if (c->e1->op != TOKidentifier)
                         error("identifier or call expression expected for attribute");
                     if (c->arguments)
                         for (int unsigned ai = 0; ai < c->arguments->dim; ai++)
                         {
-                            c->arguments->data[ai] =
-                                ((Expression *) c->arguments->data[ai])->semantic(sc);
+                            Expression * ea = c->arguments->tdata()[ai];
+                            ea = ea->semantic(sc);
+                            ea = ea->optimize(WANTvalue | WANTinterpret);
+                            c->arguments->tdata()[ai] = ea;
                         }
                 }
                 else
@@ -1112,16 +1139,15 @@ void PragmaDeclaration::semantic(Scope *sc)
                 new_attrs->push(e);
             }
 
-            for (unsigned i = 0; i < p_attributes_list.dim; ++i)
+            if (p_attributes)
             {
-                Expressions ** pa = (Expressions **) p_attributes_list.data[i];
-                if (*pa)
+                if (*p_attributes)
                 {
-                    *pa = (Expressions *) (*pa)->copy();
-                    (*pa)->append(new_attrs);
+                    *p_attributes = (Expressions *) (*p_attributes)->copy();
+                    (*p_attributes)->append(new_attrs);
                 }
                 else
-                    *pa = new_attrs;
+                    *p_attributes = new_attrs;
             }
         }
         goto Lnodecl;
@@ -1184,7 +1210,7 @@ void PragmaDeclaration::semantic(Scope *sc)
     {
         for (unsigned i = 0; i < decl->dim; i++)
         {
-            Dsymbol *s = (Dsymbol *)decl->data[i];
+            Dsymbol *s = decl->tdata()[i];
 
             s->semantic(sc);
         }
@@ -1192,7 +1218,7 @@ void PragmaDeclaration::semantic(Scope *sc)
 
 #if IN_GCC
     if (decl)
-        if (ident == Id::GNU_attribute)
+        if (ident == Id::GNU_attribute || ident == Id::_GNU_attribute)
             *sc = sc_save;
 #endif
     return;
@@ -1235,11 +1261,10 @@ void PragmaDeclaration::toObjFile(int multiobj)
         obj_includelib(name);
 #elif ELFOBJ || MACHOBJ
         /* The format does not allow embedded library names,
-         * The ELF format does not allow embedded library names,
          * so instead append the library name to the list to be passed
          * to the linker.
          */
-        global.params.libfiles->push((void *) name);
+        global.params.libfiles->push(name);
 #else
         error("pragma lib not supported");
 #endif
@@ -1279,7 +1304,7 @@ void PragmaDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
 
 /********************************* ConditionalDeclaration ****************************/
 
-ConditionalDeclaration::ConditionalDeclaration(Condition *condition, Array *decl, Array *elsedecl)
+ConditionalDeclaration::ConditionalDeclaration(Condition *condition, Dsymbols *decl, Dsymbols *elsedecl)
         : AttribDeclaration(decl)
 {
     //printf("ConditionalDeclaration::ConditionalDeclaration()\n");
@@ -1304,7 +1329,7 @@ int ConditionalDeclaration::oneMember(Dsymbol **ps)
     //printf("ConditionalDeclaration::oneMember(), inc = %d\n", condition->inc);
     if (condition->inc)
     {
-        Array *d = condition->include(NULL, NULL) ? decl : elsedecl;
+        Dsymbols *d = condition->include(NULL, NULL) ? decl : elsedecl;
         return Dsymbol::oneMembers(d, ps);
     }
     *ps = NULL;
@@ -1319,21 +1344,21 @@ void ConditionalDeclaration::emitComment(Scope *sc)
         AttribDeclaration::emitComment(sc);
     }
     else if (sc->docbuf)
-     {
+    {
         /* If generating doc comment, be careful because if we're inside
          * a template, then include(NULL, NULL) will fail.
          */
-        Array *d = decl ? decl : elsedecl;
+        Dsymbols *d = decl ? decl : elsedecl;
         for (unsigned i = 0; i < d->dim; i++)
-        {   Dsymbol *s = (Dsymbol *)d->data[i];
+        {   Dsymbol *s = d->tdata()[i];
             s->emitComment(sc);
         }
-     }
+    }
 }
 
 // Decide if 'then' or 'else' code should be included
 
-Array *ConditionalDeclaration::include(Scope *sc, ScopeDsymbol *sd)
+Dsymbols *ConditionalDeclaration::include(Scope *sc, ScopeDsymbol *sd)
 {
     //printf("ConditionalDeclaration::include()\n");
     assert(condition);
@@ -1342,14 +1367,14 @@ Array *ConditionalDeclaration::include(Scope *sc, ScopeDsymbol *sd)
 
 void ConditionalDeclaration::setScope(Scope *sc)
 {
-    Array *d = include(sc, NULL);
+    Dsymbols *d = include(sc, NULL);
 
     //printf("\tConditionalDeclaration::setScope '%s', d = %p\n",toChars(), d);
     if (d)
     {
        for (unsigned i = 0; i < d->dim; i++)
        {
-           Dsymbol *s = (Dsymbol *)d->data[i];
+           Dsymbol *s = d->tdata()[i];
 
            s->setScope(sc);
        }
@@ -1358,14 +1383,14 @@ void ConditionalDeclaration::setScope(Scope *sc)
 
 void ConditionalDeclaration::importAll(Scope *sc)
 {
-    Array *d = include(sc, NULL);
+    Dsymbols *d = include(sc, NULL);
 
     //printf("\tConditionalDeclaration::importAll '%s', d = %p\n",toChars(), d);
     if (d)
     {
        for (unsigned i = 0; i < d->dim; i++)
        {
-           Dsymbol *s = (Dsymbol *)d->data[i];
+           Dsymbol *s = d->tdata()[i];
 
            s->importAll(sc);
        }
@@ -1382,7 +1407,7 @@ void ConditionalDeclaration::addComment(unsigned char *comment)
 
     if (comment)
     {
-        Array *d = decl;
+        Dsymbols *d = decl;
 
         for (int j = 0; j < 2; j++)
         {
@@ -1391,7 +1416,7 @@ void ConditionalDeclaration::addComment(unsigned char *comment)
                 for (unsigned i = 0; i < d->dim; i++)
                 {   Dsymbol *s;
 
-                    s = (Dsymbol *)d->data[i];
+                    s = d->tdata()[i];
                     //printf("ConditionalDeclaration::addComment %s\n", s->toChars());
                     s->addComment(comment);
                 }
@@ -1413,7 +1438,7 @@ void ConditionalDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
         {
             for (unsigned i = 0; i < decl->dim; i++)
             {
-                Dsymbol *s = (Dsymbol *)decl->data[i];
+                Dsymbol *s = decl->tdata()[i];
 
                 buf->writestring("    ");
                 s->toCBuffer(buf, hgs);
@@ -1429,7 +1454,7 @@ void ConditionalDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
             buf->writenl();
             for (unsigned i = 0; i < elsedecl->dim; i++)
             {
-                Dsymbol *s = (Dsymbol *)elsedecl->data[i];
+                Dsymbol *s = elsedecl->tdata()[i];
 
                 buf->writestring("    ");
                 s->toCBuffer(buf, hgs);
@@ -1445,7 +1470,7 @@ void ConditionalDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
 /***************************** StaticIfDeclaration ****************************/
 
 StaticIfDeclaration::StaticIfDeclaration(Condition *condition,
-        Array *decl, Array *elsedecl)
+        Dsymbols *decl, Dsymbols *elsedecl)
         : ConditionalDeclaration(condition, decl, elsedecl)
 {
     //printf("StaticIfDeclaration::StaticIfDeclaration()\n");
@@ -1468,7 +1493,7 @@ Dsymbol *StaticIfDeclaration::syntaxCopy(Dsymbol *s)
 
 int StaticIfDeclaration::addMember(Scope *sc, ScopeDsymbol *sd, int memnum)
 {
-        //printf("StaticIfDeclaration::addMember() '%s'\n",toChars());
+    //printf("StaticIfDeclaration::addMember() '%s'\n",toChars());
     /* This is deferred until semantic(), so that
      * expressions in the condition can refer to declarations
      * in the same scope, such as:
@@ -1503,7 +1528,7 @@ void StaticIfDeclaration::setScope(Scope *sc)
 
 void StaticIfDeclaration::semantic(Scope *sc)
 {
-    Array *d = include(sc, sd);
+    Dsymbols *d = include(sc, sd);
 
     //printf("\tStaticIfDeclaration::semantic '%s', d = %p\n",toChars(), d);
     if (d)
@@ -1515,7 +1540,7 @@ void StaticIfDeclaration::semantic(Scope *sc)
 
         for (unsigned i = 0; i < d->dim; i++)
         {
-            Dsymbol *s = (Dsymbol *)d->data[i];
+            Dsymbol *s = d->tdata()[i];
 
             s->semantic(sc);
         }
@@ -1533,8 +1558,8 @@ const char *StaticIfDeclaration::kind()
 CompileDeclaration::CompileDeclaration(Loc loc, Expression *exp)
     : AttribDeclaration(NULL)
 {
-        //printf("CompileDeclaration(loc = %d)\n", loc.linnum);
-        this->loc = loc;
+    //printf("CompileDeclaration(loc = %d)\n", loc.linnum);
+    this->loc = loc;
     this->exp = exp;
     this->sd = NULL;
     this->compiled = 0;
@@ -1563,23 +1588,23 @@ int CompileDeclaration::addMember(Scope *sc, ScopeDsymbol *sd, int memnum)
 
 void CompileDeclaration::compileIt(Scope *sc)
 {
-     //printf("CompileDeclaration::compileIt(loc = %d)\n", loc.linnum);
+    //printf("CompileDeclaration::compileIt(loc = %d)\n", loc.linnum);
     exp = exp->semantic(sc);
     exp = resolveProperties(sc, exp);
     exp = exp->optimize(WANTvalue | WANTinterpret);
-    if (exp->op != TOKstring)
+    StringExp *se = exp->toString();
+    if (!se)
     {   exp->error("argument to mixin must be a string, not (%s)", exp->toChars());
     }
     else
     {
-    StringExp *se = (StringExp *)exp;
-    se = se->toUTF8(sc);
-    Parser p(sc->module, (unsigned char *)se->string, se->len, 0);
-    p.loc = loc;
-    p.nextToken();
-    decl = p.parseDeclDefs(0);
-    if (p.token.value != TOKeof)
-        exp->error("incomplete mixin declaration (%s)", se->toChars());
+        se = se->toUTF8(sc);
+        Parser p(sc->module, (unsigned char *)se->string, se->len, 0);
+        p.loc = loc;
+        p.nextToken();
+        decl = p.parseDeclDefs(0);
+        if (p.token.value != TOKeof)
+            exp->error("incomplete mixin declaration (%s)", se->toChars());
     }
 }
 
