@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2010 by Digital Mars
+// Copyright (c) 1999-2011 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -198,7 +198,7 @@ ClassDeclaration::ClassDeclaration(Loc loc, Identifier *id, BaseClasses *basecla
     }
 
     com = 0;
-    isauto = 0;
+    isscope = 0;
     isabstract = 0;
     isnested = 0;
     vthis = NULL;
@@ -218,7 +218,7 @@ Dsymbol *ClassDeclaration::syntaxCopy(Dsymbol *s)
     cd->storage_class |= storage_class;
 
     cd->baseclasses->setDim(this->baseclasses->dim);
-    for (int i = 0; i < cd->baseclasses->dim; i++)
+    for (size_t i = 0; i < cd->baseclasses->dim; i++)
     {
         BaseClass *b = (BaseClass *)this->baseclasses->data[i];
         BaseClass *b2 = new BaseClass(b->type->syntaxCopy(), b->protection);
@@ -230,8 +230,7 @@ Dsymbol *ClassDeclaration::syntaxCopy(Dsymbol *s)
 }
 
 void ClassDeclaration::semantic(Scope *sc)
-{   int i;
-
+{
     //printf("ClassDeclaration::semantic(%s), type = %p, sizeok = %d, this = %p\n", toChars(), type, sizeok, this);
     //printf("\tparent = %p, '%s'\n", sc->parent, sc->parent ? sc->parent->toChars() : "");
     //printf("sc->stc = %x\n", sc->stc);
@@ -246,11 +245,12 @@ void ClassDeclaration::semantic(Scope *sc)
 
     if (!sc)
         sc = scope;
-        if (!parent && sc->parent && !sc->parent->isModule())
-            parent = sc->parent;
+    if (!parent && sc->parent && !sc->parent->isModule())
+        parent = sc->parent;
+
+    type = type->semantic(loc, sc);
+    handle = type;
 
-        type = type->semantic(loc, sc);
-        handle = type;
     if (!members)                       // if forward reference
     {   //printf("\tclass '%s' is forward referenced\n", toChars());
         return;
@@ -270,12 +270,13 @@ void ClassDeclaration::semantic(Scope *sc)
         scx = scope;            // save so we don't make redundant copies
         scope = NULL;
     }
+    unsigned dprogress_save = Module::dprogress;
+#ifdef IN_GCC
     if (attributes)
         attributes->append(sc->attributes);
     else
         attributes = sc->attributes;
-        unsigned dprogress_save = Module::dprogress;
-#ifdef IN_GCC
+
     methods.setDim(0);
 #endif
 
@@ -285,7 +286,7 @@ void ClassDeclaration::semantic(Scope *sc)
     }
 
     // Expand any tuples in baseclasses[]
-    for (i = 0; i < baseclasses->dim; )
+    for (size_t i = 0; i < baseclasses->dim; )
     {   BaseClass *b = (BaseClass *)baseclasses->data[i];
         b->type = b->type->semantic(loc, sc);
         Type *tb = b->type->toBasetype();
@@ -375,7 +376,7 @@ void ClassDeclaration::semantic(Scope *sc)
 
     // Treat the remaining entries in baseclasses as interfaces
     // Check for errors, handle forward references
-    for (i = (baseClass ? 1 : 0); i < baseclasses->dim; )
+    for (size_t i = (baseClass ? 1 : 0); i < baseclasses->dim; )
     {   TypeClass *tc;
         BaseClass *b;
         Type *tb;
@@ -480,7 +481,7 @@ void ClassDeclaration::semantic(Scope *sc)
 
         // Inherit properties from base class
         com = baseClass->isCOMclass();
-        isauto = baseClass->isauto;
+        isscope = baseClass->isscope;
         vthis = baseClass->vthis;
     }
     else
@@ -497,7 +498,7 @@ void ClassDeclaration::semantic(Scope *sc)
     {
         interfaceSemantic(sc);
 
-        for (i = 0; i < members->dim; i++)
+        for (size_t i = 0; i < members->dim; i++)
         {
             Dsymbol *s = (Dsymbol *)members->data[i];
             s->addMember(sc, this, 1);
@@ -512,7 +513,7 @@ void ClassDeclaration::semantic(Scope *sc)
             if (storage_class & STCstatic)
                 error("static class cannot inherit from nested class %s", baseClass->toChars());
             if (toParent2() != baseClass->toParent2())
-                {
+            {
                 if (toParent2())
                 {
                     error("is nested within %s, but super class %s is nested within %s",
@@ -543,9 +544,9 @@ void ClassDeclaration::semantic(Scope *sc)
                     if (ad)
                         t = ad->handle;
                     else if (fd)
-                    {   AggregateDeclaration *ad = fd->isMember2();
-                        if (ad)
-                            t = ad->handle;
+                    {   AggregateDeclaration *ad2 = fd->isMember2();
+                        if (ad2)
+                            t = ad2->handle;
                         else
                         {
                             t = new TypePointer(Type::tvoid);
@@ -562,15 +563,18 @@ void ClassDeclaration::semantic(Scope *sc)
         }
     }
 
-    if (storage_class & (STCauto | STCscope))
-        isauto = 1;
+    if (storage_class & STCauto)
+        error("storage class 'auto' is invalid when declaring a class, did you mean to use 'scope'?");
+    if (storage_class & STCscope)
+        isscope = 1;
     if (storage_class & STCabstract)
         isabstract = 1;
 
     sc = sc->push(this);
-    sc->stc &= ~(STCfinal | STCauto | STCscope | STCstatic |
-                 STCabstract | STCdeprecated);
+    sc->stc &= STCsafe | STCtrusted | STCsystem;
+#if IN_GCC
     sc->attributes = NULL;
+#endif
     sc->parent = this;
     sc->inunion = 0;
 
@@ -601,25 +605,25 @@ void ClassDeclaration::semantic(Scope *sc)
     }
     structsize = sc->offset;
     Scope scsave = *sc;
-    int members_dim = members->dim;
+    size_t members_dim = members->dim;
     sizeok = 0;
 
     /* Set scope so if there are forward references, we still might be able to
      * resolve individual members like enums.
      */
-    for (i = 0; i < members_dim; i++)
+    for (size_t i = 0; i < members_dim; i++)
     {   Dsymbol *s = (Dsymbol *)members->data[i];
         /* There are problems doing this in the general case because
          * Scope keeps track of things like 'offset'
          */
         if (s->isEnumDeclaration() || (s->isAggregateDeclaration() && s->ident))
-    {
+        {
             //printf("setScope %s %s\n", s->kind(), s->toChars());
             s->setScope(sc);
         }
     }
 
-    for (i = 0; i < members_dim; i++)
+    for (size_t i = 0; i < members_dim; i++)
     {   Dsymbol *s = (Dsymbol *)members->data[i];
         s->semantic(sc);
     }
@@ -695,7 +699,7 @@ void ClassDeclaration::semantic(Scope *sc)
 #endif
 
     // Allocate instance of each new interface
-    for (i = 0; i < vtblInterfaces->dim; i++)
+    for (size_t i = 0; i < vtblInterfaces->dim; i++)
     {
         BaseClass *b = (BaseClass *)vtblInterfaces->data[i];
         unsigned thissize = PTRSIZE;
@@ -744,7 +748,7 @@ void ClassDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
         if (baseclasses->dim)
             buf->writestring(" : ");
     }
-    for (int i = 0; i < baseclasses->dim; i++)
+    for (size_t i = 0; i < baseclasses->dim; i++)
     {
         BaseClass *b = (BaseClass *)baseclasses->data[i];
 
@@ -758,7 +762,7 @@ void ClassDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
         buf->writenl();
         buf->writeByte('{');
         buf->writenl();
-        for (int i = 0; i < members->dim; i++)
+        for (size_t i = 0; i < members->dim; i++)
         {
             Dsymbol *s = (Dsymbol *)members->data[i];
 
@@ -794,7 +798,7 @@ int ClassDeclaration::isBaseOf2(ClassDeclaration *cd)
     if (!cd)
         return 0;
     //printf("ClassDeclaration::isBaseOf2(this = '%s', cd = '%s')\n", toChars(), cd->toChars());
-    for (int i = 0; i < cd->baseclasses->dim; i++)
+    for (size_t i = 0; i < cd->baseclasses->dim; i++)
     {   BaseClass *b = (BaseClass *)cd->baseclasses->data[i];
 
         if (b->base == this || isBaseOf2(b->base))
@@ -807,41 +811,60 @@ int ClassDeclaration::isBaseOf2(ClassDeclaration *cd)
  * Determine if 'this' is a base class of cd.
  */
 
-int ClassDeclaration::isBaseOf(ClassDeclaration *cd, target_ptrdiff_t *poffset)
+int ClassDeclaration::isBaseOf(ClassDeclaration *cd, int *poffset)
 {
     //printf("ClassDeclaration::isBaseOf(this = '%s', cd = '%s')\n", toChars(), cd->toChars());
     if (poffset)
         *poffset = 0;
     while (cd)
     {
-        if (this == cd->baseClass)
-            return 1;
-
         /* cd->baseClass might not be set if cd is forward referenced.
          */
         if (!cd->baseClass && cd->baseclasses->dim && !cd->isInterfaceDeclaration())
         {
-            cd->error("base class is forward referenced by %s", toChars());
+            cd->semantic(NULL);
+            if (!cd->baseClass)
+                cd->error("base class is forward referenced by %s", toChars());
         }
 
+        if (this == cd->baseClass)
+            return 1;
+
         cd = cd->baseClass;
     }
     return 0;
 }
 
+/*********************************************
+ * Determine if 'this' has complete base class information.
+ * This is used to detect forward references in covariant overloads.
+ */
+
+int ClassDeclaration::isBaseInfoComplete()
+{
+    if (!baseClass)
+        return ident == Id::Object;
+    for (size_t i = 0; i < baseclasses->dim; i++)
+    {   BaseClass *b = (BaseClass *)baseclasses->data[i];
+        if (!b->base || !b->base->isBaseInfoComplete())
+            return 0;
+    }
+    return 1;
+}
+
 Dsymbol *ClassDeclaration::search(Loc loc, Identifier *ident, int flags)
 {
     Dsymbol *s;
     //printf("%s.ClassDeclaration::search('%s')\n", toChars(), ident->toChars());
 
-    if (scope)
-        {       Scope *sc = scope;
+    if (scope && !symtab)
+    {   Scope *sc = scope;
         sc->mustsemantic++;
         semantic(sc);
         sc->mustsemantic--;
     }
 
-    if (!members || !symtab || scope)
+    if (!members || !symtab)
     {
         error("is forward referenced when looking for '%s'", ident->toChars());
         //*(char*)0=0;
@@ -853,9 +876,7 @@ Dsymbol *ClassDeclaration::search(Loc loc, Identifier *ident, int flags)
     {
         // Search bases classes in depth-first, left to right order
 
-        int i;
-
-        for (i = 0; i < baseclasses->dim; i++)
+        for (size_t i = 0; i < baseclasses->dim; i++)
         {
             BaseClass *b = (BaseClass *)baseclasses->data[i];
 
@@ -991,7 +1012,7 @@ int ClassDeclaration::isAbstract()
 {
     if (isabstract)
         return TRUE;
-    for (int i = 1; i < vtbl.dim; i++)
+    for (size_t i = 1; i < vtbl.dim; i++)
     {
         FuncDeclaration *fd = ((Dsymbol *)vtbl.data[i])->isFuncDeclaration();
 
@@ -1068,11 +1089,11 @@ Dsymbol *InterfaceDeclaration::syntaxCopy(Dsymbol *s)
 }
 
 void InterfaceDeclaration::semantic(Scope *sc)
-{   int i;
-
+{
     //printf("InterfaceDeclaration::semantic(%s), type = %p\n", toChars(), type);
     if (inuse)
         return;
+
     if (!sc)
         sc = scope;
     if (!parent && sc->parent && !sc->parent->isModule())
@@ -1080,6 +1101,7 @@ void InterfaceDeclaration::semantic(Scope *sc)
 
     type = type->semantic(loc, sc);
     handle = type;
+
     if (!members)                       // if forward reference
     {   //printf("\tinterface '%s' is forward referenced\n", toChars());
         return;
@@ -1097,11 +1119,12 @@ void InterfaceDeclaration::semantic(Scope *sc)
         scx = scope;            // save so we don't make redundant copies
         scope = NULL;
     }
-
+#if IN_GCC
     if (attributes)
         attributes->append(sc->attributes);
     else
         attributes = sc->attributes;
+#endif
 
     if (sc->stc & STCdeprecated)
     {
@@ -1109,7 +1132,7 @@ void InterfaceDeclaration::semantic(Scope *sc)
     }
 
     // Expand any tuples in baseclasses[]
-    for (i = 0; i < baseclasses->dim; )
+    for (size_t i = 0; i < baseclasses->dim; )
     {   BaseClass *b = (BaseClass *)baseclasses->data[0];
         b->type = b->type->semantic(loc, sc);
         Type *tb = b->type->toBasetype();
@@ -1130,7 +1153,7 @@ void InterfaceDeclaration::semantic(Scope *sc)
     }
 
     // Check for errors, handle forward references
-    for (i = 0; i < baseclasses->dim; )
+    for (size_t i = 0; i < baseclasses->dim; )
     {   TypeClass *tc;
         BaseClass *b;
         Type *tb;
@@ -1193,11 +1216,11 @@ void InterfaceDeclaration::semantic(Scope *sc)
         vtbl.push(this);                // leave room at vtbl[0] for classinfo
 
     // Cat together the vtbl[]'s from base interfaces
-    for (i = 0; i < interfaces_dim; i++)
+    for (size_t i = 0; i < interfaces_dim; i++)
     {   BaseClass *b = interfaces[i];
 
         // Skip if b has already appeared
-        for (int k = 0; k < i; k++)
+        for (size_t k = 0; k < i; k++)
         {
             if (b == interfaces[k])
                 goto Lcontinue;
@@ -1205,12 +1228,12 @@ void InterfaceDeclaration::semantic(Scope *sc)
 
         // Copy vtbl[] from base class
         if (b->base->vtblOffset())
-        {   int d = b->base->vtbl.dim;
+        {   size_t d = b->base->vtbl.dim;
             if (d > 1)
             {
                 vtbl.reserve(d - 1);
-                for (int j = 1; j < d; j++)
-                    vtbl.push(b->base->vtbl.data[j]);
+                for (size_t j = 1; j < d; j++)
+                    vtbl.push((Dsymbol *)b->base->vtbl.data[j]);
             }
         }
         else
@@ -1222,14 +1245,16 @@ void InterfaceDeclaration::semantic(Scope *sc)
         ;
     }
 
-    for (i = 0; i < members->dim; i++)
+    for (size_t i = 0; i < members->dim; i++)
     {
         Dsymbol *s = (Dsymbol *)members->data[i];
         s->addMember(sc, this, 1);
     }
 
     sc = sc->push(this);
+#if IN_GCC
     sc->attributes = NULL;
+#endif
     sc->parent = this;
     if (isCOMinterface())
         sc->linkage = LINKwindows;
@@ -1237,7 +1262,7 @@ void InterfaceDeclaration::semantic(Scope *sc)
     structalign = sc->structalign;
     sc->offset = PTRSIZE * 2;
     inuse++;
-    for (i = 0; i < members->dim; i++)
+    for (size_t i = 0; i < members->dim; i++)
     {
         Dsymbol *s = (Dsymbol *)members->data[i];
         s->semantic(sc);
@@ -1260,7 +1285,7 @@ void InterfaceDeclaration::semantic(Scope *sc)
  *      1       is a base
  */
 
-int InterfaceDeclaration::isBaseOf(ClassDeclaration *cd, target_ptrdiff_t *poffset)
+int InterfaceDeclaration::isBaseOf(ClassDeclaration *cd, int *poffset)
 {
     unsigned j;
 
@@ -1297,7 +1322,7 @@ int InterfaceDeclaration::isBaseOf(ClassDeclaration *cd, target_ptrdiff_t *poffs
 }
 
 
-int InterfaceDeclaration::isBaseOf(BaseClass *bc, target_ptrdiff_t *poffset)
+int InterfaceDeclaration::isBaseOf(BaseClass *bc, int *poffset)
 {
     //printf("%s.InterfaceDeclaration::isBaseOf(bc = '%s')\n", toChars(), bc->base->toChars());
     for (unsigned j = 0; j < bc->baseInterfaces_dim; j++)
@@ -1308,11 +1333,14 @@ int InterfaceDeclaration::isBaseOf(BaseClass *bc, target_ptrdiff_t *poffset)
         {
             if (poffset)
             {   *poffset = b->offset;
+                if (j && bc->base->isInterfaceDeclaration())
+                    *poffset = OFFSET_RUNTIME;
             }
             return 1;
         }
         if (isBaseOf(b, poffset))
-        {
+        {   if (j && poffset && bc->base->isInterfaceDeclaration())
+                *poffset = OFFSET_RUNTIME;
             return 1;
         }
     }
@@ -1321,6 +1349,22 @@ int InterfaceDeclaration::isBaseOf(BaseClass *bc, target_ptrdiff_t *poffset)
     return 0;
 }
 
+/*********************************************
+ * Determine if 'this' has clomplete base class information.
+ * This is used to detect forward references in covariant overloads.
+ */
+
+int InterfaceDeclaration::isBaseInfoComplete()
+{
+    assert(!baseClass);
+    for (size_t i = 0; i < baseclasses->dim; i++)
+    {   BaseClass *b = (BaseClass *)baseclasses->data[i];
+        if (!b->base || !b->base->isBaseInfoComplete ())
+            return 0;
+    }
+    return 1;
+}
+
 /****************************************
  * Determine if slot 0 of the vtbl[] is reserved for something else.
  * For class objects, yes, this is where the ClassInfo ptr goes.
@@ -1386,10 +1430,9 @@ BaseClass::BaseClass(Type *type, enum PROT protection)
  *      by base classes)
  */
 
-int BaseClass::fillVtbl(ClassDeclaration *cd, Array *vtbl, int newinstance)
+int BaseClass::fillVtbl(ClassDeclaration *cd, FuncDeclarations *vtbl, int newinstance)
 {
     ClassDeclaration *id = base;
-    int j;
     int result = 0;
 
     //printf("BaseClass::fillVtbl(this='%s', cd='%s')\n", base->toChars(), cd->toChars());
@@ -1397,7 +1440,7 @@ int BaseClass::fillVtbl(ClassDeclaration *cd, Array *vtbl, int newinstance)
         vtbl->setDim(base->vtbl.dim);
 
     // first entry is ClassInfo reference
-    for (j = base->vtblOffset(); j < base->vtbl.dim; j++)
+    for (size_t j = base->vtblOffset(); j < base->vtbl.dim; j++)
     {
         FuncDeclaration *ifd = ((Dsymbol *)base->vtbl.data[j])->isFuncDeclaration();
         FuncDeclaration *fd;
