@@ -27,49 +27,51 @@
 struct Identifier;
 struct Type;
 struct Expression;
-#ifdef _DH
 struct HdrGenState;
-#endif
 
 
 struct EnumDeclaration : ScopeDsymbol
 {   /* enum ident : memtype { ... }
      */
-    Type *type;			// the TypeEnum
-    Type *memtype;		// type of the members
+    Type *type;                 // the TypeEnum
+    Type *memtype;              // type of the members
 
-#if V1
-    integer_t maxval;
-    integer_t minval;
-    integer_t defaultval;	// default initializer
+#if DMDV1
+    dinteger_t maxval;
+    dinteger_t minval;
+    dinteger_t defaultval;      // default initializer
 #else
     Expression *maxval;
     Expression *minval;
-    Expression *defaultval;	// default initializer
-
-    Scope *scope;		// !=NULL means context to use
+    Expression *defaultval;     // default initializer
+#endif
+#if IN_GCC
+    Expressions * attributes;   // GCC decl/type attributes
 #endif
-    Expressions * attributes; // GCC decl/type attributes
     int isdeprecated;
+    int isdone;                 // 0: not done
+                                // 1: semantic() successfully completed
 
     EnumDeclaration(Loc loc, Identifier *id, Type *memtype);
     Dsymbol *syntaxCopy(Dsymbol *s);
+    void semantic0(Scope *sc);
     void semantic(Scope *sc);
     int oneMember(Dsymbol **ps);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     Type *getType();
     const char *kind();
-#if V2
+#if DMDV2
     Dsymbol *search(Loc, Identifier *ident, int flags);
 #endif
-    int isDeprecated();			// is Dsymbol deprecated?
+    int isDeprecated();                 // is Dsymbol deprecated?
 
     void emitComment(Scope *sc);
+    void toJsonBuffer(OutBuffer *buf);
     void toDocBuffer(OutBuffer *buf);
 
     EnumDeclaration *isEnumDeclaration() { return this; }
 
-    void toObjFile(int multiobj);			// compile to .obj file
+    void toObjFile(int multiobj);                       // compile to .obj file
     void toDebug();
     int cvMember(unsigned char *p);
 
@@ -89,6 +91,7 @@ struct EnumMember : Dsymbol
     const char *kind();
 
     void emitComment(Scope *sc);
+    void toJsonBuffer(OutBuffer *buf);
     void toDocBuffer(OutBuffer *buf);
 
     EnumMember *isEnumMember() { return this; }
